@@ -280,9 +280,9 @@ def angle_difference(v1, v2, direction: bool = False) -> float:
 
     dot = np.dot(v1n, v2n)
 
-    if dot > 1:
+    if dot >= 1:
         ang = 0
-    if dot < -1:
+    elif dot <= -1:
         ang = 180
     else:
         ang = math.acos(dot)*180/math.pi
@@ -293,6 +293,61 @@ def angle_difference(v1, v2, direction: bool = False) -> float:
     return ang
 
 
+def relative_angular_weighting(vs, vList: list, nList: list):
+    '''
+    Computes the relative contributions of the segments in vList to vs using
+    relative angular weighting, which attributes less weight to fixel
+    perpendicular to the streamline segment.
+
+    Parameters
+    ----------
+    vs : 1-D array
+        Segment vector
+    vList : list
+        List of the k vectors corresponding to each fiber population
+    nList : list
+        List of the null k vectors
+
+    Returns
+    -------
+    ang_coef : list
+        List of the k coefficients
+
+    '''
+
+    K = len(vList)
+
+    if K == 1:
+        return [1]
+
+    if K-sum(nList) <= 1:
+        return [1-i for i in list(map(int, nList))]
+
+    angle_diffList = []
+
+    for i, v in enumerate(vList):
+        if nList[i]:
+            angle_diffList.append(0)
+        else:
+            angle_diffList.append(angle_difference(vs, v))
+
+    sum_diff = np.sum(angle_diffList)
+
+    ang_coef = []
+
+    for i, angle_diff in enumerate(angle_diffList):
+        if nList[i]:
+            ang_coef.append(0)
+        elif sum_diff == K*90:    # Else divides by 0
+            ang_coef.append(1/K)
+        else:
+            coef = (min(90, sum_diff)-angle_diff)/(min(90, sum_diff)
+                                                   * (K-sum(nList))-sum_diff)
+            ang_coef.append(coef)
+
+    return ang_coef
+
+
 def angular_weighting(vs, vList: list, nList: list):
     '''
     Computes the relative contributions of the segments in vList to vs using
@@ -337,6 +392,8 @@ def angular_weighting(vs, vList: list, nList: list):
     for i, angle_diff in enumerate(angle_diffList):
         if nList[i]:
             ang_coef.append(0)
+        elif sum_diff == K*90:    # Else divides by 0
+            ang_coef.append(1/K)
         else:
             coef = 1-angle_diff/sum_diff
             coef = coef/(K-1-sum(nList))
@@ -552,7 +609,8 @@ def tensor_to_peak(t):
 
 
 def get_fixel_weight_MF(trk_file: str, MF_dir: str, Patient: str, K: int = 2,
-                        method: str = 'ang', streamList: list = []):
+                        method: str = 'ang', streamList: list = [],
+                        speed_up: bool = False):
     '''
     Get the fixel weights from a tract specified in trk_file and the peaks
     obtained from Microsrcuture Fingerprinting.
@@ -622,12 +680,13 @@ def get_fixel_weight_MF(trk_file: str, MF_dir: str, Patient: str, K: int = 2,
 
             fList.append(f)
 
-    return get_fixel_weight(trk, tList, method, streamList, fList)
+    return get_fixel_weight(trk, tList, method, streamList, fList,
+                            speed_up=speed_up)
 
 
 def get_fixel_weight_DIAMOND(trk_file: str, DIAMOND_dir: str, Patient: str,
                              K: int = 2, method: str = 'ang',
-                             streamList: list = []):
+                             streamList: list = [], speed_up: bool = False):
     '''
     Get the fixel weights from a tract specified in trk_file and the tensors
     obtained from DIAMOND.
@@ -703,7 +762,8 @@ def get_fixel_weight_DIAMOND(trk_file: str, DIAMOND_dir: str, Patient: str,
 
             fList.append(fk)
 
-    return get_fixel_weight(trk, tList, method, streamList, fList)
+    return get_fixel_weight(trk, tList, method, streamList, fList,
+                            speed_up=speed_up)
 
 
 def get_fixel_weight(trk, tList: list, method: str = 'ang',
@@ -756,7 +816,7 @@ def get_fixel_weight(trk, tList: list, method: str = 'ang',
 
     '''
 
-    assert method in ['ang', 'cfo', 'vol'], ("Unknown method : "+method)
+    assert method in ['ang', 'cfo', 'vol', 'raw'], ("Unknown method : "+method)
 
     phi_maps = {}
     K = len(tList)
@@ -821,6 +881,8 @@ def get_fixel_weight(trk, tList: list, method: str = 'ang',
                                       + ").")
                     coefList = fraction_weighting(
                         (x, y, z), vList, nList, fList)
+                elif method == 'raw':
+                    coefList = relative_angular_weighting(vs, vList, nList)
                 else:   # Angular weighting
                     coefList = angular_weighting(vs, vList, nList)
 
@@ -933,7 +995,7 @@ def tract_to_streamlines(trk) -> list:
 
 
 def get_streamline_weights(trk, tList: list,
-                           method_list: list = ['vol', 'cfo', 'ang'],
+                           method_list: list = ['vol', 'cfo', 'ang', 'raw'],
                            streamline_number: int = 0, fList: list = []):
     '''
 
@@ -1023,6 +1085,8 @@ def get_streamline_weights(trk, tList: list,
                                       + ").")
                     coefList = fraction_weighting(
                         (x, y, z), vList, nList, fList)
+                elif method == 'raw':
+                    coefList = relative_angular_weighting(vs, vList, nList)
                 else:   # Angular weighting
                     coefList = angular_weighting(vs, vList, nList)
 
@@ -1046,7 +1110,7 @@ def get_streamline_weights(trk, tList: list,
 
 
 def plot_streamline_metrics(trk, tList: list, metric_maps: list,
-                            method_list: list = ['vol', 'cfo', 'ang'],
+                            method_list: list = ['vol', 'cfo', 'ang', 'raw'],
                             streamline_number: int = 0, fList: list = [],
                             segment_wise: bool = True, groundTruth_map=None,
                             barplot: bool = True):
